'Insert Linked Frames (by Abe Hendin)
'InsertLinkedFrames.csc
'Revision 1.1
'
'Copyright 2001-2002 by Abe Hendin <ventura@yourspeed.com>.
'Public Domain.
'Initial release July 18, 2001.
'Revised August 6, 2001, to end on actual last page of chapter,
'following text reflow after insertion of frames on preceding pages.
'
'Revision 1.1, August 11, 2002: added options to create a specific
'number of frames and to create frames mid-series, and added much
'expanded error handling.
' 
'Please let me know if you have ideas to improve on it, or if
'you discover bugs.
'
'This script inserts frames linked to the currently selected
'frame on subsequent pages of the current chapter, one frame per
'page, and tags them with the frame tag of the selected frame.
'
'I wrote this to speed the creation of a series of linked frames
'for manual placement of footnotes/endnotes. If you create a frame
'tag that includes size and position info (uncheck "omit from
'tag" on the frame properties Placement tab), you can create a
'frame in the same position on each page. This is what I've done
'for footnote frames, which can then be resized as necessary.
'For pages with no footnotes, delete the frame created on that
'page.
'
'To easily delete unneeded empty frames, use my "DeleteLinkedFrames"
'script.
'
'NOTES:
'Be sure to place a text file in the selected frame before running the
'script; if you create a series of blank frames first, the text placed
'into the first frame WILL NOT flow automatically through subsequent
'frames, and you'll have to place the text manually in each frame,
'which defeats the essential purpose of this script.
' 
'Also, the script won't work unless you first set the view to
'"Zoom to Actual Size" (1:1) (press F4). Unfortunately, the
'.ViewZoomLayout command is mostly broken in Ventura 8.559.
'
'The script isn't yet smart enough to handle two linked frames on the
'same page; starting it from the first of multiple linked frames on a
'single page may cause unpredictable results.

WITHOBJECT "CorelVentura.Automation.10"

Dim FrameTag As String
Dim BasicText As String, MaxPossibleText As String
Dim CurrView As Integer, CurrPN As Integer, NextLinkedPN As Integer
Dim InterveningPages As Integer
Dim ChapLastPage As Integer
Dim FrameInd As Integer, FC As Integer
Dim retval As Integer, retDialog As Integer
Dim NumOfFrames As Integer, MaxFrames As Integer, MaxIFrames As Integer
Dim FramesToCreate As Integer, FramesCreated As Integer
Dim NextLinkedCheck As Boolean

'tests for Page Layout view (required) and attempts to set 1:1
CurrView = .CurrentView()
.ViewZoomLayout 3 'this command is partially broken in V8.559

'sets variable to test for invalid frame selection
FrameInd = .CurrentFrameIndex()

'gets frame tag name of selected frame
FrameTag = ""
On Error Resume Next 'in case no frame is selected
FrameTag = .FormatGetFrameTag()

'tests to see if any frames can be created
CurrPN = .CurrentPageNumber(, 3) 'relative to chapter
ChapLastPage = .ChapterPageCount()
MaxFrames = ChapLastPage - CurrPN

'tests for existing linked frames
.FrameNextLinked
NextLinkedPN = .CurrentPageNumber(, 3)
MaxIFrames = 0
If CurrPN = NextLinkedPN Then
	NextLinkedCheck = False 'no linked frames exist
Else
	NextLinkedCheck = True 'there is at least one linked frame
	InterveningPages = NextLinkedPN - CurrPN
	.FramePrevLinked 'return to selected frame
	If InterveningPages = 1 Then 'cannot create a frame
		MaxFrames = 0
	ElseIf InterveningPages >= 2 Then
		MaxIFrames = InterveningPages - 1
		MaxFrames = MaxIFrames
	EndIf
EndIf

If CurrView = 0 And FrameTag <> "" And MaxFrames <> 0 And FrameInd => 0 Then
	If NextLinkedCheck = True And MaxIFrames > 0 Then
		MaxPossibleText = "The current frame is followed by a linked frame on Page " + \\
		Ltrim(Str(NextLinkedPN)) + " (absolute). You can therefore create a maximum of " + \\
		Ltrim(Str(MaxIFrames)) + " linked frame(s) here."
	ElseIf NextLinkedCheck = False Then
		MaxPossibleText = "There are no linked frames following the currently " + \\
			"selected frame. You can create as many linked frames as you like, " + \\
			"up to the end of the chapter."
	EndIf
	BasicText = "Leave the Number of Frames to Create set to 0 to create " + \\
		"the maximum possible number of linked frames at this location." + \\
		Chr(13) + "This script will fail unless the view is first set to 1:1 (F4)."
BEGIN DIALOG Dialog1 205, 116, "Create How Many Linked Frames?"
	SPINCONTROL  134, 71, 29, 12, NumOfFrames
	TEXT  8, 33, 190, 27, BasicText
	TEXT  39, 72, 92, 12, "Number of Frames to Create:"
	PUSHBUTTON  31, 91, 82, 14, "Create &Linked Frames"	 '3
	PUSHBUTTON  127, 91, 46, 14, "&Cancel"	 '4
	TEXT  8, 2, 190, 27, MaxPossibleText
END DIALOG

	retDialog = Dialog(Dialog1)
	If retDialog = 4 Or retDialog = 2 Then 'user hit Cancel
		Stop
	Else 'user hit Create Linked Frames
		FramesCreated = 0
		Do
			'get starting frame location info
			CurrPN = .CurrentPageNumber(, 3) 'relative to chapter; reset at each round

			'set page number of last page of chapter; reset at each round
			ChapLastPage = .ChapterPageCount()
			MaxFrames = ChapLastPage - CurrPN
				If MaxIFrames > 0 Then 'if linked frames exist, resets to max intervening
					MaxFrames = MaxIFrames - FramesCreated
				EndIf
			
			If NumOfFrames = 0 Then 'user wants max # of frames
				FramesToCreate = MaxFrames
			ElseIf NumOfFrames >= MaxFrames Then 'NoF is more than max
				FramesToCreate = MaxFrames
				If FramesCreated = NumOfFrames Then 'if FCreated = # requested, done
					FramesToCreate = 0
				EndIf
			ElseIf NumOfFrames > 0 And NumOfFrames < MaxFrames Then
				FramesToCreate = NumOfFrames - FramesCreated
				If FramesToCreate < 0 Then 'test number for negative value
					FramesToCreate = 0
				EndIf
			EndIf

			If FramesToCreate = 0 And FramesCreated > 0 Then
				retval = MessageBox("The last linked frame in the " + \\
					"series has been created.", "Done Creating Frames", 64)
				Stop 'we're done; exit the script
			EndIf

			Do Until FramesToCreate = 0
				' Inserts a linked frame on the next page and tags it
				.FrameContinueTextFlow 35300, 35300, 63500, 63500, .Before = FALSE, \\
					.TargetPageNumber = -1
				.FormatSetFrameTag FrameTag

				' Changes view to next page and selects the frame just inserted for repeat action
				.ViewGotoPage .GoWhere = 3
				FC = .FrameCount()
				.FrameFirst .FrameIndex = FC
	
				FramesCreated = FramesCreated + 1
				FramesToCreate = FramesToCreate - 1
			Loop
		Loop
	EndIf
ElseIf CurrView <> 0 Then
	retval = MessageBox("For this script to execute, you must first put" + \\
		Chr(13) + "Ventura into Page Layout view and select a frame." + \\
		Chr(13) + "Place the text file you want to flow through the" + \\
		Chr(13) + "series in the selected frame *before* running the script.", \\
		"Error: Not in PL View; No Frame Selected", 48)
ElseIf FrameTag = "" And CurrView = 0 Then
	retval = MessageBox("For this script to execute, you must first select a frame." + \\
		Chr(13) + "Place the text file you want to flow through the series" + \\
		Chr(13) + "in the selected frame *before* running the script.", \\
		"Error: No Frame Selected", 48)
ElseIf MaxFrames = 0 And FramesCreated = 0 And NextLinkedCheck = False \\
	And FrameInd => 0 Then
	retval = MessageBox("Because the current frame is on the last page" + \\
		Chr(13) + "of the chapter, no more frames can be created.", \\
		"Cannot Create Frames", 48)
ElseIf NextLinkedCheck = True And MaxFrames = 0 Then
	retval = MessageBox("Because the current frame is followed by a linked" + \\
		Chr(13) + "frame on the next page, no frames can be created here.", \\
		"Cannot Create Frames", 48)
ElseIf FrameInd < 0 Then
	retval = MessageBox("Cannot start with a Base Page, Header, Footer, or " + \\
		Chr(13) + "auto-footnote frame. Please select a drawn frame" + \\
		Chr(13) + "and try again.", "Error: Invalid Frame Selected", 48)
EndIf

END WITHOBJECT
